import { useState, useEffect } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

/** @deprecated */
function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var eventListenerMouseEvent = Object.freeze({
    mousedown: true,
    mouseup: true,
    mousemove: true,
    wheel: true,
});
var isDefined = function (x) { return typeof x !== 'undefined'; };
var isDeltaNegative = function (delta) { return (isDefined(delta) ? delta < 0 : undefined); };
var isDeltaPositive = function (delta) { return (isDefined(delta) ? delta > 0 : undefined); };
var generateMouseEventPosition = function (event) { return ({
    client: { x: event.clientX, y: event.clientY },
    page: { x: event.pageX, y: event.pageY },
    screen: { x: event.screenX, y: event.screenY },
}); };
var LEFT_BUTTONS = [1, 3, 5, 7];
var RIGHT_BUTTONS = [2, 3, 6, 7];
var MIDDLE_BUTTONS = [4, 5, 6, 7];
var generateMouseEventButtons = function (event) { return ({
    left: LEFT_BUTTONS.includes(event.buttons),
    right: RIGHT_BUTTONS.includes(event.buttons),
    middle: MIDDLE_BUTTONS.includes(event.buttons),
}); };
var generateMouseEventWheel = function (event) { return ({
    deltaX: event.deltaX,
    left: isDeltaNegative(event.deltaX),
    right: isDeltaPositive(event.deltaX),
    deltaY: event.deltaY,
    up: isDeltaNegative(event.deltaY),
    down: isDeltaPositive(event.deltaY),
    deltaZ: event.deltaZ,
    out: isDeltaNegative(event.deltaZ),
    in: isDeltaPositive(event.deltaZ),
    deltaMode: event.deltaMode,
}); };
var generateMouseEventModifierKeys = function (event) { return ({
    alt: event.altKey,
    ctrl: event.ctrlKey,
    meta: event.metaKey,
    shift: event.shiftKey,
}); };
var mouseEventHandlerFactory = function (setMouse) { return (function (event) {
    setMouse({
        position: generateMouseEventPosition(event),
        movement: { x: event.movementX, y: event.movementY },
        buttons: generateMouseEventButtons(event),
        keyboard: generateMouseEventModifierKeys(event),
        wheel: generateMouseEventWheel(event),
    });
}); };
var mouseEvents = Object.keys(eventListenerMouseEvent);
var doMouseEventListener = function (action, eventListeners, event, handler) {
    if (!eventListeners[event]) {
        return;
    }
    document[action](event, handler);
};
var registerMouseEventListener = function (eventListeners, event, handler) { return doMouseEventListener('addEventListener', eventListeners, event, handler); };
var unregisterMouseEventListener = function (eventListeners, event, handler) { return doMouseEventListener('removeEventListener', eventListeners, event, handler); };
var useMouse = (function (eventListenerOptions) {
    if (eventListenerOptions === void 0) { eventListenerOptions = {
        mousedown: true,
        mouseup: true,
        mousemove: true,
        wheel: true,
    }; }
    var _a = useState(null), mouse = _a[0], setMouse = _a[1];
    useEffect(function () {
        var eventListeners = mouseEvents.reduce(function (ac, mouseEvent) {
            var result = ac;
            result[mouseEvent] = !!eventListenerOptions[mouseEvent];
            return result;
        }, {});
        var handleMouseEvent = mouseEventHandlerFactory(setMouse);
        mouseEvents.forEach(function (mouseEvent) { return (registerMouseEventListener(eventListeners, mouseEvent, handleMouseEvent)); });
        return function () {
            mouseEvents.forEach(function (mouseEvent) { return (unregisterMouseEventListener(eventListeners, mouseEvent, handleMouseEvent)); });
        };
    }, __spreadArrays(mouseEvents.map(function (mouseEvent) { return !!eventListenerOptions[mouseEvent]; })));
    return mouse;
});

export { useMouse as default };
